import { Injectable } from '@angular/core';
import { invoke } from '@tauri-apps/api';
import { from, map, tap } from 'rxjs';
import { Plan, PlanInterface } from '../_models/plan';

@Injectable({
  providedIn: 'root'
})
export class PlanService {

  currentPlan?: Plan;
  planStore: string[] = [];

  constructor() {
    this.getPreviousPlans();
  }

  loadPlan(path: string) {
    return from(invoke<PlanInterface>('load_plan', { path })).pipe(
      map(plan => {
        this.currentPlan = new Plan(plan);
        return plan
      })
    );
  }

  loadPlanNoSave(path: string) {
    return from(invoke<PlanInterface>('load_plan', { path })).pipe(
      map(plan => {
        this.currentPlan = new Plan(plan);
        this.currentPlan.setPath(path);
        return plan
      })
    );
  }

  loadBasePlan() {
    return from(invoke<PlanInterface>('base_plan'));
  }

  savePlan(path: string, plan: Plan) {
    plan.plan.forEach(elem => {
      if (!elem.notes) { elem.notes = "" }
    });

    return from(invoke<boolean>('save_plan', {
      path,
      plan: {
        plan: plan.plan,
        current: plan.current
      },
    })).subscribe(status => {
    });
  }

  getPreviousPlans() {
    from(invoke<string[]>('load_stored_plans')).subscribe(plans => this.planStore = plans);
  }

  loadPrevious(name: string) {
    return from(invoke<PlanInterface>('load_previous', { name })).pipe(tap(plan => {
      console.log("previous loaded: ", plan);
      this.currentPlan = new Plan(plan);
      this.currentPlan.setPrevious(name);
    }));
  }
}
